// udStFiLrCorrWare
// J. J. Weimer
// History: $Revision: 849 $ on $Date: 2008-07-10 17:09:47 -0500 (Thu, 10 Jul 2008) $
//
// Procedure files to read a CorrWare file and store the waves
//
// Access to Package
//
// -> udStFiLrCorrWare(udFL)
//		STRUCTURE udFLStandardStructure &udFL
//
// see the structure definition and User's Manual

#pragma rtGlobals=1
#pragma IgorVersion=6.03
#pragma version=1.0

#include "LinkDisplay"
#include "udFLStandardFunctions", versions>=2.0

// **********************************************
// Globals and Localizations

Static Constant kVersion = 1.0

// **********************************************
// Initialize (Zero + Query + Check for PlugIns)

Static Function udFLCorrWareInititalize(udFL)
	STRUCT udFLStandardStructure &udFL
	
	udFLZeroStructure(udFL)
	udFLCorrWareQuery(udFL)
	return 0
end

// **********************************************
// Query

Static Function udFLCorrWareQuery(udFL)
	STRUCT udFLStandardStructure &udFL
		
	udFL.mimetype = "TEXT"
	udFL.extensions = ".cor;"
	udFL.loadModes += "Manual;"
	udFL.procModes += "i(E)/t;"
	udFL.itemsType = 4			// returns waves
	udFL.itemsDim[p] = p*0
	udFL.itemsDim[0]=-1			// waves are nominally unscaled numeric
	udFL.setNames=(2+8)		// filename+number or prefix+number
//	udFL.userData = "Prefix+Column#;"
	udFLCorrWareInitPlugIns(udFL)
	return 0	
end

// **********************************************
// Check for PlugIns

Static Function udFLCorrWareInitPlugIns(udFL)
	STRUCT udFLStandardStructure &udFL

	udFLCheckForPlugIn("udFLPlugInLinkDisplay",udFL)
	return 0
end

// **********************************************
// File Loader

Function udStFiLrCorrWare(udFL)
	STRUCT udFLStandardStructure &udFL

	variable refNum, ic, nt
	string pathStr, wavesLoaded, twList
	
	switch(udFL.userLCtrl)
		case -1:		// initialize		
			udFLCorrWareInititalize(udFL)		
			return 0
		case 0:		// query
			udFLCorrWareQuery(udFL)
			return 0
		case 1:		// auto
			if (strlen(udFL.fileList)==0)
				udFLSetError(udFL,"No files in the selection list!",-1)
				return -1
			endif
			break
		case 2:		// manual
			Open/C="????"/T=".cor"/D/R/M="Select the CorrWare file" refNum
			if (strlen(S_filename)!=0)
				udFL.filelist = S_filename + ";"
			else
				udFLSetError(udFL,"No file selected",-1)
				return -1
			endif
			break
	endswitch
	
	if (udFL.reportCtrl >= 1)
		print "LOADING CORR WARE DATA"
		print "... assuring file paths"
	endif
	if (udFLAssureFilePaths(udFL)<0)
		return -1
	endif
	
	if (udFL.reportCtrl >= 1)
		print "... setting data folder location"
	endif
	string cdf = GetDataFolder(1)
	udFLSetDataFolder(udFL)
	if (strlen(udFL.pathStr)!=0)
		pathStr = udFL.pathStr
		NewPath/O/Q tmpCorrWaredir, pathStr
	endif
	
	if (udFL.reportCtrl >= 1)
		print "... loading"
	endif
		
	nt = ItemsInList(udFL.fileList)
	for (ic=0;ic<nt;ic+=1)
			
		WavesLoaded = LoadCorrWareFile(ic,udFL)
		
		if (strlen(WavesLoaded)!=0)
				
			WavesLoaded = ProcessCorrWareWaves(WavesLoaded, udFL)
			WavesLoaded = RenameCorrWareWaves(ic,WavesLoaded, udFL)
//			WavesLoaded = DoUserDataCorrWareWaves(WavesLoaded, udFL)
			LinkCorrWareWaves(WavesLoaded,udFL)
			udFL.itLst.WAVES += WavesLoaded
				
		endif
		
	endfor
	
	udFL.itemsCount = ItemsInList(udFL.itLst.WAVES)
	
	SetDataFolder $cdf
	
	if (udFL.reportCtrl == 2)
		print "Structure @ Loader:", udFL
	endif
	
	return 0
end

// **********************************************
// Load A File

Static Function/S LoadCorrWareFile(ic,udFL)
	variable ic
	STRUCT udFLStandardStructure &udFL
	
	string fileStr=StringFromList(ic,udFL.fileList)
	
	if (udFL.reportCtrl >= 1)
		print "... file ", fileStr
	endif
	if (strlen(WinList("CorrFileReader",";","WIN:16"))!=0)
		KillWindow CorrFileReader
	endif
	if (strlen(udFL.pathStr)==0)
		OpenNotebook/V=0/K=3/R/N=CorrFileReader fileStr
	else
		OpenNotebook/V=0/K=3/R/P=tmpCorrWaredir/N=CorrFileReader fileStr
	endif
	
	return (ParseCorrWareNotebook(fileStr))
end

Function/S ParseCorrWareNotebook(fileStr)
	string fileStr

	string wList = "", theLine="", theData="", theHeader=""
	string ev, iv, tv
	variable dstart, npoints, ic
	
	Notebook CorrFileReader selection={startofFile,startOfFile}
	Notebook CorrFileReader findText={"End Comments",4}
	GetSelection notebook, CorrFileReader, 1
	dStart = V_startParagraph + 1
	Notebook CorrFileReader selection={startOfFile,endOfParagraph}
	GetSelection notebook, CorrFileReader, 2
	theHeader = S_selection	
	
	Notebook CorrFileReader selection={startofFile,startOfFile}
	Notebook CorrFileReader findText={"Data Points:",4}
	
	GetSelection notebook, CorrFileReader, 1
	Notebook CorrFileReader selection={startOfParagraph,endOfParagraph}
	GetSelection notebook, CorrFileReader, 2
	SplitString/E="Data Points: [ ]*([0-9]*)" S_selection, theLine
	
	npoints = str2num(theLine)
	make/O/N=(npoints) eWave, iWave, tWave
	
	Notebook CorrFileReader selection={(dStart,0),endofFile}
	GetSelection notebook, CorrFileReader, 2
	theData = S_selection
	
	for (ic=0;ic<npoints;ic+=1)
		theLine = StringFromList(ic,theData,"\r")
		SplitString/E="^(.+)\\t+(.*)\\t+(.+)" theLine, eV, iV, tV
		eWave[ic] = str2num(eV)
		iWave[ic] = str2num(iV)
		tWave[ic] = str2num(tV)
	endfor
	
	KillWindow CorrFileReader
	
	sprintf theLine "Header%s" CleanUpName(fileStr[0,25],0)
	string/G $theLine
	SVAR hstr = $theLine
	hstr = theHeader
	
	wList = "eWave;iWave;tWave;"
	return wList
end

// **********************************************
// Process the Waves

Static Function/S ProcessCorrWareWaves(WavesLoaded,udFL)
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL
	
	variable ni=0, xs, xe
	string xwave
		
	switch((udFL.userPCtrl)-1)
		case 1:	// i(E)/t
			ni = ItemsInList(WavesLoaded)
			if (ni<3)
				udFLSetError(udFL,"Could not scale only two waves",0)
				break
			endif
			if (udFL.reportCtrl >= 1)
				print "... scaling E and i waves based on data of equal increment in t wave"
			endif
			xwave = StringFromList(2,WavesLoaded)
			wave nwave=$xwave
			xs = nwave[0]
			xe = nwave[numpnts(nwave)-1]
			killwaves/Z nwave
			xwave = StringFromList(0,WavesLoaded)
			wave nwave = $xwave
			SetScale/I x, xs, xe, nwave			
			xwave = StringFromList(1,WavesLoaded)
			wave nwave = $xwave
			SetScale/I x, xs, xe, nwave			
			WavesLoaded = "eWave;iWave;"
			break
	endswitch
	return WavesLoaded
end

// **********************************************
// Apply User Data to Waves
//
// [...] wave selection

Static Function/S DoUserDataCorrWareWaves(WavesLoaded, udFL)
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL	
	return WavesLoaded
end

// **********************************************
// Rename Waves

Static Function/S RenameCorrWareWaves(ic,WavesLoaded,udFL)
	variable ic
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL

	string fileStr = StringFromList(ic,udFL.fileList)
	string theOne, theList
		
	switch(udFL.setNames)
		case 1:
			theOne = StringFromList(0,WavesLoaded)
			theList=udFLRenameInput(theOne,"CWEwave")
			theOne = StringFromList(1,WavesLoaded)
			theList+=udFLRenameInput(theOne,"CWiwave")
			theOne = StringFromList(2,WavesLoaded)
			if (strlen(theOne)!=0)
				theList+=udFLRenameInput(theOne,"CWtwave")
			endif
			WavesLoaded=theList
			break
		case 2:
			udFL.namePrefix = ReplaceString(" ",ParseFilePath(3,fileStr,":",0,0),"")
			if (udFL.reportCtrl >= 1)
				print "... renaming wave(s) by filename ", udFL.namePrefix, " + number"
			endif
			WavesLoaded=udFLRenameInput(WavesLoaded,udFL.namePrefix)
			break
		case 8:
			if (udFL.reportCtrl >= 1)
				print "... renaming wave(s) by prefix ", udFL.namePrefix, " + number"
			endif
			WavesLoaded=udFLRenameInput(WavesLoaded,udFL.namePrefix)
			break
	endswitch
	return WavesLoaded
end

// **********************************************
// (possibly) Link the Waves

Static Function LinkCorrWareWaves(WavesLoaded,udFL)
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL

	string wList = udFL.itLst.WAVES
	string theOne
	
	if (udFL.userPCtrl==2)
		theOne = StringFromList(0,WavesLoaded)
		wave xwave = $theOne
		theOne = StringFromList(1,WavesLoaded)
		LinkDisplay#Link(xwave,theOne)
		return 0
	endif

#if(exists("udFLPlugInLinkDisplay")==6)
	udFL.userPCtrl+=-2
	udFL.itLst.WAVES=WavesLoaded
	udFLPlugInLinkDisplay(udFL)
	udFL.itLst.WAVES=wList
	udFL.userPCtrl+=2
#endif

	return 0
end

// **********************************************
// Support Functions

Static Function/S ExpandString(theStr, [sep])
	string theStr, sep
	
	string xs, xe
	variable ns, ne, ic

	if(ParamIsDefault(sep))
		sep = ","
	endif
	
	SplitString/E="(.+)-(.+)" theStr, xs,xe
	
	if (strlen(xs)==0)
		return (theStr + sep)
	endif
	ns = str2num(xs)
	ne = str2num(xe)
	
	theStr = ""
	for(ic=ns;ic<(ne+1);ic+=1)
		theStr += num2str(ic) + sep
	endfor
	return theStr
end
